include masm32rt.inc    ; http://masm32.com/
include gdiplus.inc     ;
includelib gdiplus.lib  ;
include gdip.inc
.686p
.mmx
.xmm

include \macros\unicode.inc        ; SmplMath-Package: 
include \macros\SmplMath\math.inc  ; http://sourceforge.net/projects/smplmath/
                                   ; http://www.masm32.com/board/index.php?topic=17095.0

FPS EQU 24                  ; frames per second      Unit: 1/s
fSlvRegConst FLAKE_SIZE,5   ; size of snow flake in. Unit: mm
fSlvRegConst V_SNOW,4       ; velocity of snow fall. Unit: mm/s

;/* random value from 0...1 */
fSlvRegExpr <urand>,0,<rv(badRandom256)/255>

;/* random value from -1...0...1 */
fSlvRegExpr <irand>,0,<(127-rv(badRandom256))/128>

WndProc proto hWnd:HWND,uMsg:UINT,wParam:WPARAM,lParam:LPARAM

FLAKE struct
    x       REAL4   ?   ;/* x position of flake                  */
    t       SDWORD  ?   ;/* begin of aniamtion: GetTickCount()   */
    f       REAL4   ?   ;/* horizontal oscillation frequenc      */
    v       REAl4   ?   ;/* speed of fall: mm/s                  */
    A       REAL4   ?   ;/* amplitude of horizontal oscillation  */
    yOff    REAL4   ?   ;/* vertical pixel offset                */
    _size   REAl4   ?   ;/* flake size in mm                     */
    dPhi    REAL4   ?   ;/* phsae shift: radian                  */
    f_rot   REAl4   ?   ;/* frequency of flake rotation: 1/s     */
    phi_rot REAl4   ?   ;/* rotation angle of flake: degree      */
FLAKE ends
PFLAKE typedef ptr FLAKE

WND_DATA struct
    hdc         HDC     ?
    TimerID     DWORD   ?
    bmpSnow     PVOID   ?   ;/* image object: snow flake          */
    nFlakes     DWORD   ?   ;/* number of flakes                  */
    pFlakes     PFLAKE  ?   ;/* pointer to array of FLAKE-structs */
WND_DATA ends

.code
main proc 
LOCAL wcex:WNDCLASSEX
LOCAL msg:MSG
LOCAL gsi:GdiplusStartupInput
LOCAL gtkn:PULONG

    mov gsi.GdiplusVersion,1
    mov gsi.DebugEventCallback,0
    mov gsi.SuppressBackgroundThread,0
    mov gsi.SuppressExternalCodecs,0
    invoke GdiplusStartup,ADDR gtkn,ADDR gsi,0

    mov wcex.hInstance,rv(GetModuleHandle,0)
    mov wcex.cbSize,SIZEOF WNDCLASSEX
    mov wcex.style, CS_HREDRAW or CS_VREDRAW
    mov wcex.lpfnWndProc, OFFSET WndProc
    mov wcex.cbClsExtra,NULL
    mov wcex.cbWndExtra,SIZEOF WND_DATA
    mov wcex.hbrBackground,0
    mov wcex.lpszMenuName,NULL
    mov wcex.lpszClassName,chr$("Win32GDI+,SmplMath")
    mov wcex.hIcon,rv(LoadIcon,NULL,IDI_APPLICATION)
    mov wcex.hIconSm,eax
    mov wcex.hCursor,rv(LoadCursor,NULL,IDC_ARROW)
    invoke RegisterClassEx, ADDR wcex
    mov esi,rv(CreateWindowEx,NULL,wcex.lpszClassName,"win32,GDI+,SmplMath",WS_OVERLAPPEDWINDOW,CW_USEDEFAULT,CW_USEDEFAULT,600,400,NULL,NULL,wcex.hInstance,NULL)
    invoke ShowWindow,esi,SW_SHOWNORMAL
    invoke UpdateWindow,esi

    .while 1
        invoke GetMessage,ADDR msg,NULL,0,0
        .break .if !eax || eax == -1
        invoke TranslateMessage, ADDR msg
        invoke DispatchMessage, ADDR msg
    .endw
    
    invoke GdiplusShutdown,gtkn
    invoke ExitProcess,msg.wParam

main endp

badRandom256 proc uses esi edi
LOCAL qw:QWORD
    rdtsc
    and eax,0ffh
    push eax
    mov esi,eax
    xor edi,edi
    .while esi
        xor edi,rv(GetTickCount)
        dec esi
    .endw
    rdtsc
    pop edx
    mul edx
    xor eax,edi
    push eax
    invoke QueryPerformanceCounter,ADDR qw
    pop eax
    xor eax,DWORD ptr qw[0]
    and eax,0ffh

    ret
    
badRandom256 endp

WndProc proc uses ebx esi edi hWnd:HWND,uMsg:UINT,wParam:WPARAM,lParam:LPARAM
LOCAL fSlvTLS()
LOCAL ps:PAINTSTRUCT
LOCAL graphics:PVOID
LOCAL t:REAL8
LOCAL bitmap:PVOID
LOCAL y:REAL4
LOCAL x:REAL4
LOCAL rect:RECT
LOCAL rectf:RectF
LOCAL imgAttr:PVOID
LOCAL matrix:PVOID
LOCAL pixpmmY:REAL4
LOCAL pixpmmX:REAL4

    .if uMsg == WM_CLOSE
        invoke PostQuitMessage,NULL
    .elseif uMsg == WM_DESTROY
        invoke DeleteObject,rv(GetWindowLong,hWnd,WND_DATA.hdc)
        invoke GdipDisposeImage,rv(GetWindowLong,hWnd,WND_DATA.bmpSnow)
        invoke DeleteAtom,rv(GetWindowLong,hWnd,WND_DATA.TimerID)
    .elseif uMsg == WM_CREATE
        
        mov esi,rv(GetDC,rv(GetDesktopWindow))
        mov ebx,rv(CreateCompatibleDC,esi)
        mov edi,rv(GetSystemMetrics,SM_CXSCREEN)
        invoke DeleteObject,rv(SelectObject,ebx,rv(CreateCompatibleBitmap,esi,edi,rv(GetSystemMetrics,SM_CYSCREEN)))
        invoke ReleaseDC,rv(GetDesktopWindow),esi
        invoke SetWindowLong,hWnd,WND_DATA.hdc,ebx
        
        invoke GdipCreateFromHDC,ebx,ADDR graphics
        invoke GdipGraphicsClear,graphics,ColorsBlack       
        invoke GdipDeleteGraphics,graphics      
        mov edx,rv(GetWindowLong,hWnd,GWL_HINSTANCE)
        invoke GdipCreateBitmapFromResource,edx,100,ADDR bitmap
        invoke SetWindowLong,hWnd,WND_DATA.bmpSnow,bitmap
        
        invoke GetClientRect,hWnd,ADDR rect
        invoke GdipGetImageDimension,bitmap,ADDR rectf._Width,ADDR rectf.Height
        
        ;/* pixel per millimeter screen */
        fSlv pixpmmX = rv(GetDeviceCaps,rv(GetDC,0),LOGPIXELSX)*0.0393700787
        fSlv pixpmmY = rv(GetDeviceCaps,rv(GetDC,0),LOGPIXELSY)*0.0393700787
        
        ;/* calculate number of snow flakes */
        fSlv esi = rect.right/(FLAKE_SIZE*pixpmmX)*1.5
        invoke SetWindowLong,hWnd,WND_DATA.nFlakes,esi
        mov eax,esi
        imul eax,eax,SIZEOF FLAKE
        mov edi,alloc(eax)
        invoke SetWindowLong,hWnd,WND_DATA.pFlakes,edi
        mov ebx,rv(GetTickCount)
        .while esi
            ;/* random initial values */
            
            ;/* constants for x and y position */
            fSlv [edi].FLAKE.x =  urand()
            fSlv [edi].FLAKE.A = FLAKE_SIZE*0.25+FLAKE_SIZE*irand()*0.125
            fSlv [edi].FLAKE.v = V_SNOW+V_SNOW*irand()*0.25
            fSlv [edi].FLAKE.f = 0.2 + irand()*0.1
            fSlv [edi].FLAKE.dPhi = urand()*2*pi
            fSlv [edi].FLAKE._size = FLAKE_SIZE+FLAKE_SIZE*irand()*0.3
            fSlv [edi].FLAKE.yOff = -urand()*rect.bottom
            mov [edi].FLAKE.t,ebx
            
            ;/* constants for flake rotation */
            fSlv [edi].FLAKE.f_rot = [edi].FLAKE.f / 2
            fSlv [edi].FLAKE.phi_rot = urand()*45
            add edi,SIZEOF FLAKE
            dec esi
        .endw
            
        mov ebx,rv(AddAtom,"atom: drawing timer")
        invoke SetWindowLong,hWnd,WND_DATA.TimerID,ebx
        invoke SetTimer,hWnd,ebx,1000/FPS,0

    .elseif uMsg == WM_TIMER
        .if wParam == rv(GetWindowLong,hWnd,WND_DATA.TimerID)
            mov ebx,rv(GetDC,rv(GetDesktopWindow))
            fSlv pixpmmX = rv(GetDeviceCaps,ebx,LOGPIXELSX)*0.0393700787
            fSlv pixpmmY = rv(GetDeviceCaps,ebx,LOGPIXELSY)*0.0393700787
            invoke ReleaseDC,rv(GetDesktopWindow),ebx
            
            ;/* attributes are needed for transparent drawing of flake */
            invoke GdipCreateImageAttributes,ADDR imgAttr
            invoke GdipSetImageAttributesColorKeys,imgAttr,ColorAdjustTypeDefault,TRUE,0ff00ffh,0ff00ffh
            invoke GdipCreateMatrix,ADDR matrix
            
            ;/* get graphics object */
            mov ebx,rv(GetWindowLong,hWnd,WND_DATA.hdc)
            invoke GdipCreateFromHDC,ebx,ADDR graphics
            invoke GdipSetSmoothingMode,graphics,SmoothingModeHighQuality
            invoke GdipGraphicsClear,graphics,ColorsBlack       
            
            invoke GetClientRect,hWnd,ADDR rect
            
            mov esi,rv(GetWindowLong,hWnd,WND_DATA.pFlakes)
            mov edi,rv(GetWindowLong,hWnd,WND_DATA.nFlakes)
            mov ebx,rv(GetWindowLong,hWnd,WND_DATA.bmpSnow)
            invoke GdipGetImageDimension,ebx,ADDR rectf._Width,ADDR rectf.Height
            
            .while edi
                ;/* get runtime for current snow flake */
                fSlv t = (rv(GetTickCount)-[esi].FLAKE.t) * 1.E-3
                
                ;/* get x and y position of flake - this is the basic formula */
                fSlv y = [esi].FLAKE.v * t * pixpmmY + [esi].FLAKE.yOff
                fSlv x = [esi].FLAKE.A * pixpmmX * sin(2*pi*[esi].FLAKE.f*t+[esi].FLAKE.dPhi) + [esi].FLAKE.x*rect.right
                
                ;/* rotate flake randomly around its center */
                invoke GdipSetMatrixElements,matrix,FP4(1.0),FP4(0.0),FP4(0.0),FP4(1.0),FP4(0.0),FP4(0.0)
                invoke GdipTranslateMatrix,matrix,@fSlv4(-x-[esi].FLAKE._size*pixpmmX*0.5),@fSlv4(-y-[esi].FLAKE._size*pixpmmY*0.5),MatrixOrderAppend
                invoke GdipRotateMatrix,matrix,@fSlv4([esi].FLAKE.phi_rot*sin(2*pi*[esi].FLAKE.f_rot*t)),MatrixOrderAppend
                invoke GdipTranslateMatrix,matrix,@fSlv4(x+[esi].FLAKE._size*pixpmmX*0.5),@fSlv4(y+[esi].FLAKE._size*pixpmmY*0.5),MatrixOrderAppend
                invoke GdipSetWorldTransform,graphics,matrix
                
                ;/* draw snow flake */              
                invoke GdipDrawImageRectRect,graphics,ebx,x,y,@fSlv4([esi].FLAKE._size*pixpmmX),@fSlv4([esi].FLAKE._size*pixpmmY),0,0,rectf._Width,rectf.Height,UnitPixel,imgAttr,0,0
                
                ;/* reinitalize snow flake if needed */
                .if fGT(y,rect.bottom)
                    fSlv [esi].FLAKE.x =  urand()
                    fSlv [esi].FLAKE.A = FLAKE_SIZE*0.25+FLAKE_SIZE*irand()*0.125
                    fSlv [esi].FLAKE.v = V_SNOW+V_SNOW*irand()*0.25
                    fSlv [esi].FLAKE.f = 0.2 + irand()*0.1
                    fSlv [esi].FLAKE.dPhi = urand()*2*pi
                    fSlv [esi].FLAKE._size = FLAKE_SIZE+FLAKE_SIZE*irand()*0.3
                    fSlv [esi].FLAKE.yOff = -(FLAKE_SIZE)*pixpmmY
                    mov [esi].FLAKE.t,rv(GetTickCount)
                    fSlv [esi].FLAKE.f_rot = [esi].FLAKE.f / 2
                    fSlv [esi].FLAKE.phi_rot = urand()*45
                .endif
                
                add esi,SIZEOF FLAKE
                dec edi
            .endw
            
            invoke GdipDeleteMatrix,matrix
            invoke GdipDisposeImageAttributes,imgAttr
            invoke GdipDeleteGraphics,graphics
            invoke InvalidateRect,hWnd,0,0
        .endif
    .elseif uMsg == WM_PAINT
        invoke BeginPaint,hWnd,ADDR ps
        mov ebx,rv(GetWindowLong,hWnd,WND_DATA.hdc)
        mov edx,ps.rcPaint.right
        sub edx,ps.rcPaint.left
        mov ecx,ps.rcPaint.bottom
        sub ecx,ps.rcPaint.top
        invoke BitBlt,ps.hdc,ps.rcPaint.left,ps.rcPaint.top,edx,ecx,ebx,ps.rcPaint.left,ps.rcPaint.top,SRCCOPY
        invoke EndPaint,hWnd,ADDR ps    
    .else
        invoke DefWindowProc,hWnd,uMsg,wParam,lParam
        ret
    .endif

    xor eax,eax
    ret
WndProc endp
end main